-- Ksztette : Vgh Tams 2006.11.15.
-- 
--Tervezzen egy egyszer szmll ramkrt, amely egy nyomgomb (BTN0) 
--megnyomsnak hatsra nvel egy 4 bites szmot.  
--A nyomgomb megnyomsi idejt mrni kell (min. 500 ms) s prellmentesteni kell.

--A prell azt jelenti, hogy a nyomgomb (mechanikus kontaktus) megnyomsakor ill.
--elengedsekor zavar impulzusok keletkeznek, mieltt stabilizldna a jel.
--Ennek a zavar impulzus csomagnak a hossza kisebb 20ms-nl egy mikrokapcsolnl.
--A nyomgomb jele min. 40..100 ms hossz.  

--Ez a vltozat egyetlen VHDL modulbl ll, ami tartalmazza ugyanazt a strukturlt
--tervet, amit a demo_a s a demo_b.

--Ez a vltozat is a rendszerterv (blokksma) alapjn kszl, ppgy mint a demo_a s a demo_b
--vltozat. 

--Ennek a mdszernek ugyanaz az elnye, mint a rajzolt blokksms vltozatnak, azaz
--a terv strukturlt, de a modulok nem tesztelhetk egymstl fggetlenl!!!!

--Egy bonyolult strukturlt terv brmely modulja lehet ilyen felpts,azaz  
--egy modulban hasznlhatunk tbb processzt s ms konkurrens utastst.

--Ez a vltozat biztostja a legjobban a terv hordozhatsgt, mivel egyetlen VHDL-bl ll.



library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity gombdetektor is
	port ( clk50     : in    std_logic; --50MHz-es f rajel
          nyomogomb : in    std_logic; --BTN0-s nyomgomb
          rst       : in    std_logic; --aszinkron trls
          digit     : out   std_logic_vector (3 downto 0);--htszegmense kijelz digit eng. 
          led       : out   std_logic; --LED0
          seg       : out   std_logic_vector (7 downto 0));--htszegmense kijelz segmens vezrls 
			 
	  TYPE allapotok IS (megnyomasra_var,elengedesre_var);-- az FSM llapotai		 
			 
end gombdetektor;

architecture Behavioral of gombdetektor is
   signal d          : std_logic_vector (3 downto 0);--hexa szamll, htszegmenses kijelzs
   signal x20ms_out  : std_logic;--20ms-os mintavteli id jelzse
   signal gomb_esemeny    : std_logic_vector (1 downto 0);-- gombnyoms esemny jelzse 
	
	signal x20ms_count	: std_logic_vector(31 downto 0) ;	-- prellmentest oszt
	
	signal fifo : std_logic_vector(1 downto 0) ;--FIFO regiszter a nyomgombhoz 
	
	signal stopper : std_logic_vector(31 downto 0);-- real time ra gombnyoms idhz
   signal allapot: allapotok; -- automata (FSM) llapotvltozja
	
begin



--begin dekad_kij komponens
digit	 <= "1110";-- csak a jobb szls kijelzt engedlyezem a 4 kzl
  seg <= "00000011" when d = "0000" else
         "10011111" when d = "0001" else
         "00100101" when d = "0010" else
         "00001101" when d = "0011" else
         "10011001" when d = "0100" else
         "01001001" when d = "0101" else
         "01000001" when d = "0110" else
         "00011111" when d = "0111" else
         "00000001" when d = "1000" else
         "00011001" when d = "1001" else	
			"00010001" when d = "1010" else	
	      "11000001" when d = "1011" else   
         "01100011" when d = "1100" else  
         "10000101" when d = "1101" else	
         "01100001" when d = "1110" else  
         "01110001"  ;	
--end dekad_kij komponens	



--begin oszto_20ms komponens
process (clk50, rst)
begin
if rst = '1' then
     x20ms_count <= (others => '0') ;	
	  x20ms_out <= '0' ;	-- 

elsif clk50'event and clk50 = '1' then -- f rajel felfut le 
	x20ms_count <= x20ms_count + 1 ;	 --ketyeg az ra

	if x20ms_count = 1000000-1 then  -- 50 MHz-et kell leosztani  20 ms-ra
		x20ms_out <= '1' ;	--20ms-os kimenetet aktivlom (1 rajel ideig fog csak tartani!)
		x20ms_count <= (others => '0') ;
	else 
		x20ms_out <= '0' ; 
		  --csak 1 rajel ideig tarthat a kiadott parancs (ez most 1 bites parancs!)
	end if ;


end if ;
end process ;
--end oszto_20ms komponens


--begin prellment komponens
process (clk50, rst)

begin
 if rst = '1' 
    then
	   fifo <= (others => '0') ;	--resetben vagyok, ezrt a szlo minden bitje 0
	   gomb_esemeny <= (others => '0') ; -- nincs esemny jelzs
	 elsif clk50'event and clk50 = '1' 
         then -- f rajel felfut le esemny
               if x20ms_out = '1' --mintavtelezs parancs rkezett 
					     then 
	                 --prellmentests mintavteli idpontja : 20 ms-onknt
                      fifo <= fifo(0) &  nyomogomb ;--gombnyomst belptetem (mintavtelezem)
                      gomb_esemeny(0) <= (fifo(1) xor fifo(0)) and fifo(0) ;--megnyoms esemny trtnt
			             gomb_esemeny(1) <= (fifo(1) xor fifo(0)) and (not fifo(0)) ;--elengeds esemny trtnt
                    else
				          gomb_esemeny <= (others => '0') ; -- gomb esemny jelzs vge
				          					
               end if;

   	
 end if ;

end process ;
--end prellment komponens


--begin fsm komponens
process (clk50, rst)

begin
 if rst = '1' then --inicializls
   --aszinkron resetben alaphelyzetbe kell lltani a modul vltozit, ill. kimeneti portjait
                       stopper <= (others => '0') ;
                           led <= '0'  ;
	                	        d <= (others => '0')  ;
	                    allapot <= megnyomasra_var ;
	           elsif clk50'event and clk50 = '1' 
                        then -- f rajel felfut le van
                              if stopper < 5 * 50000000 then  
										      -- az idmrst 5 sec-nl lelltom (ez legyen nagyobb a max. mrend idnl
                                    stopper <= stopper +1;    -- hogy ne csorduljon tl az ra 
                              end if;

                            case allapot is --az llapottl fggen kell vizsglni az esemnyeket
                                --s vgrehajtani mveleteket ill. szksg szerint llapotvltst
										  
                              	when megnyomasra_var => 	
												 if gomb_esemeny = "01" then --gomb megnyoms esemny jelzs rkezett
												       led <= '1' ; --gy ltom a gomb megnyomst
												   stopper <= (others => '0') ;
								         	   allapot <=  elengedesre_var;
					                      end if;
	
	                              when elengedesre_var =>
				                         if gomb_esemeny = "10"  then ----gomb elengeds esemny jelzs rkezett
					                             led <= '0' ; --most engedtem el a gombot, a kijelzst ezrt kikapcsolom
					                             if (stopper >= 25000000) 
														      then --500 ms-nl hosszabb a gombnyoms hossza, elfogadom
							                               d <= d + 1; 
								                    end if;
							                       allapot <=  megnyomasra_var; --megyek a kvetkez gombnyomsra vrakoz llapotba
						       
						                   end if;
					 
	                              when others =>  null ;

                            end case ;
 
 end if ;

end process ;
--end fsm komponens




end Behavioral;

